/******************************* camera.c *******************************/

/* Provides an interface to the HP Photosmart R707 digital camera
   The camera has the following functions exposed to the micro:
   Input:  Power on sensing
   Output: On/Off
           Auto focus
           Shutter
           Video record
*/

#include <hidef.h> 
#include <mc9s12c32.h>

#include "camera.h"
#include "debug.h"
#include "timer.h"

void Camera_Init(void) 
{
   // PTAD5 is input
   ATDDIEN |= CAMERA_MASK_POWERED;
   
   // PTAD5 has a pullup
   PERAD |= CAMERA_MASK_POWERED;
   PPSAD = 0b00000000;
   
   // PTT0-PTT3 output to camera
   DDRT |= CAMERA_MASK_ON | CAMERA_MASK_AF | CAMERA_MASK_SHUTTER | CAMERA_MASK_VIDEO;
}

void Camera_TurnOn(void) 
{
   // only strobe if the camera is currently off
   if (!CAMERA_IS_POWERED()) 
   {
       DEBUG("Camera On\r\n");
               
       CAMERA_SET_ON();
       Timer_Wait1ms(100);
       CAMERA_UNSET_ON();
   }
}

void Camera_TurnOff(void) 
{
   // only strobe if the camera is currently on
   if (CAMERA_IS_POWERED()) 
   {
       DEBUG("Camera Off\r\n");
               
       CAMERA_SET_ON();
       Timer_Wait1ms(100);
       CAMERA_UNSET_ON();
   }
}

void Camera_AutoFocus(void) 
{
   DEBUG("Camera Auto focus\r\n");
   CAMERA_SET_AF();
}

void Camera_Shutter(void)
{
   DEBUG("Camera shutter\r\n");
   if (CAMERA_OUTPUT_PORT & CAMERA_MASK_AF) 
   {
      // Auto focus set first
      CAMERA_SET_SHUTTER();
      Timer_Wait1ms(50);
      CAMERA_UNSET_SHUTTER();
      CAMERA_UNSET_AF();
   }
   else 
   {
      // set auto focus, then shutter
      CAMERA_SET_AF();
      Timer_Wait1ms(100);
      CAMERA_SET_SHUTTER();
      Timer_Wait1ms(50);      
      CAMERA_UNSET_SHUTTER();
      CAMERA_UNSET_AF();
   }
}

void Camera_VideoStart(void) 
{
   DEBUG("Camera video start\r\n");
   CAMERA_SET_VIDEO();
   Timer_Wait1ms(50);
   CAMERA_UNSET_VIDEO();   
}

void Camera_VideoStop(void) 
{
   DEBUG("Camera video stop\r\n");
   CAMERA_SET_VIDEO();
   Timer_Wait1ms(50);
   CAMERA_UNSET_VIDEO();   
}
