/******************************* servo.c *******************************/

// Uses PWM module to send servo pulses
// Pulses are 1ms to 2ms every 20ms
 
#include <mc9s12c32.h>
#include "servo.h"
#include "constants.h"

/* 9S12C32 Pins
   PTT0, PTT1 concatenated
   
   Outputs on PTT1 
*/


// used to interact with duty and period registers
unsigned char *PWM_Duty8_Offset = &PWMDTY0;
unsigned short *PWM_Duty16_Offset = (unsigned short *)&PWMDTY0;
unsigned short *PWM_Period16_Offset = (unsigned short *)&PWMPER0;


// sets up the PWM registers for 20ms period and servo centered initial duty
void Servo_Init(void) 
{
   // Disable PWM module for PTT0,1; setting to 16bit mode works only when disabled   
   PWME &= ~0b00000011;
   
   // PTT0,1 used for PWM
   MODRR |= 0b00000011;
   
   // Duty On time is logic high   
   PWMPOL |= 0b00000011;
   
   // use clock A
   PWMCLK &= ~0b00000011;
   
   // left aligned PWM
   PWMCAE &= ~0b00000011; 
   
   // clock A prescaler of 8
   // for a 16bit PWM and 24MHz clock, this leads to 60000 period = 20ms
   // thus duty values of 3000 to 6000 are approprate servo values 
   PWMPRCLK = 0b00000011;
   
   // 100Hz
   //PWMPRCLK = 0b00000010;
   
   // enable PWM channel concatenation to get 16bit PWM
   PWMCTL |= 0b00010000;
  
   PWM_Period16(0, SERVO_PWM_PERIOD);
   PWM_Duty16(0, SERVO_OUT_PWM_ZERO);
  
   
   // Enable PWM module   
   PWME |= 0b00000011;
   
   /*
   // set initial period and duty cycle for 8bit PWM channels
   PWMPER0 = 255;
   PWMPER1 = 255;
  
   PWMDTY0 = 0;
   PWMDTY1 = 0;   
   */
}


// Tested with a Hitec HS-322HD
// It was able to operate at 50Hz and 100Hz, but seized at 150Hz and 200Hz.
// The total range of inputs with period=255, is duty=15 to 62, this corresponds to about 200deg of rotation 

// sets up the PWM registers for 8bit operation on PTT0,1
void Servo_Init8(void) 
{	   
   // PTT0,1 used for PWM
   MODRR |= 0b00000011;
   
   // Duty On time is logic high   
   PWMPOL |= 0b00000011;
   
   // use clock SA
   PWMCLK |= 0b00000011;
   
   // left aligned PWM
   PWMCAE &= ~0b00000011; 
   
   // clock A prescaler of 2
   // trying to achieve 100Hz output
   PWMPRCLK = 0b00000001;  
   
   // 24000000 / 2 / (2* 235) / 256 = 99.734
   // 100Hz
   PWMSCLA = 235;
   
   // 150Hz
   PWMSCLA = 156;
   
   // Enable PWM module   
   PWME |= 0b00000011;     
   
   // set initial period and duty cycle for 8bit PWM channels
   PWMPER0 = 255;
   PWMPER1 = 255;
  
   PWMDTY0 = 0;
   PWMDTY1 = 0;      
}





// sets the duty cycle of selected 8bit PWM channel
void PWM_Duty8(unsigned char channel, unsigned char value)
{
   if (channel <= 5)	 // ensure only valid channels can be set
   {        
      PWM_Duty8_Offset[channel] = value;
   }
}

// sets the duty cycle of selected 16bit PWM channel
// only two 16bit channels are available
void PWM_Duty16(unsigned char channel, unsigned short value)
{
   if (channel <= 1)	 // ensure only valid channels can be set
   {        
      PWM_Duty16_Offset[channel] = value;
   }
}

// sets the period of selected 16bit PWM channel
void PWM_Period16(unsigned char channel, unsigned short value)
{
   if (channel <= 1)	 // ensure only valid channels can be set
   {        
      PWM_Period16_Offset[channel] = value;
   }
}

// sets the pulse width of a servo channel
// this corresponds to a 16bit PWM channel
// input values are SERVO_IN_MIN to SERVO_IN_MAX
void Servo_Duty(unsigned char channel, short value)
{
   if (channel <= 1)	 // ensure only valid channels can be set
   {  
      // saturate inputs to handle out of range values
      if (value > SERVO_IN_MAX) value = SERVO_IN_MAX;
      if (value < SERVO_IN_MIN) value = SERVO_IN_MIN; 
      
      // scale to get correct servo pwm duty cycle
      
      // this simple technique ensures that the zero point
      // is handle correctly, but could lead to high or low clipping
      // a multiplicative scaling method would be more appropriate
      // but more difficult to do with integers
      value += SERVO_OUT_PWM_ZERO;      
      if (value > SERVO_OUT_PWM_MAX) value = SERVO_OUT_PWM_MAX;
      if (value < SERVO_OUT_PWM_MIN) value = SERVO_OUT_PWM_MIN;
           
      PWM_Duty16_Offset[channel] = value;
   }
}

