/******************************* spi.c *******************************/

// Provides SPI interface for use with LED drivers

#include <mc9s12c32.h>
#include "spi.h"
#include "sci.h"
#include "timer.h"
#include "utilities.h"
#include "adc.h"
#include "constants.h"



// SPI outputs and inputs simultaneously with shift registers
//  typically only one of these operations needs to occur at a time
//  so unneeded bytes are thrown away

// used to clear SPIF when necessary
unsigned char dummy;
unsigned char *mem;
unsigned char index;

unsigned int i;



// initializes SPI registers
void SPI_Init(void)
{ 
   // PTM5 - SPI Clock
   // PTM4 - SPI master output
   // PTM3 - Chip select bar - controlled separately in code
   // PTM2 - SPI slave input
 
 
   // I don't think these are required, enabling SPI should force them to be as needed
   DDRM = 0b111010;  // PTM5-3,1 = output  
   PTM = 0;          // zero out PTM registers
   DISABLE_SPI_TRANSFER;
   UNLATCH_SPI_LED;
   
   
   
   // enable SPI, set to master, use clock normally low, input on rising edge of clock
   // MSB first send
   // do not use chip select in the normal way because it needs to stay low throughout the entire operation
   //  enabling it here would make it go high after every byte
   SPICR1 = 0b01010000;
   
  
   // PTM3 is chip select, but not with CPU control, no bidirectional stuff or wait mode
   SPICR2 = 0b00000000;
  
  
   // set max allowable clock rate
    // divisor == 2, so 12MHz
   SPIBR = 0b00000000;
   
   // divisor == 4, so 6MHz
   //SPIBR = 0b00010000;
   
   // divisor == 12, so 2MHz
   //SPIBR = 0b00100001;
   
   // slow
   //SPIBR = 0b00010011;
   
   // clear SPIF
   dummy = SPISR;
   dummy = SPIDR;   
}





// sends a command byte
void IssueSPICommand(unsigned char command)
{
   SPIDR = command;
   SPI_WAIT_FOR_DATA;
   dummy = SPIDR; // clear SPIF
}


// momentarily set PTM3 high; for use in writing
void GlitchSPIChipSelect(void)
{
   DISABLE_SPI_TRANSFER; 
   Timer_Wait(6);
   ENABLE_SPI_TRANSFER;
}



// outputs a buffer through SPI
// low end first
void SPI_OutBuffer(unsigned char *pBuf, unsigned int Length) 
{
   for(i=0; i<Length; i++)
   {
      SPIDR = pBuf[i];
      SPI_WAIT_FOR_DATA;
      dummy = SPIDR;   
   }
   
   LATCH_SPI_LED;
   Timer_Wait(20);
   UNLATCH_SPI_LED;
}




// sends 16bits of LED data out the SPI
void SPI_Out_16(unsigned short val)
{			   
   mem = (unsigned char *)&val;  
    
   SPIDR = mem[0];
   SPI_WAIT_FOR_DATA;
   dummy = SPIDR;
   
   SPIDR = mem[1];
   SPI_WAIT_FOR_DATA;
   dummy = SPIDR;	     				  
  
   /*					     
   SPI_WAIT_FOR_SPACE;
   SPIDR = mem[0];     
   
   SPI_WAIT_FOR_SPACE;
   SPIDR = mem[1];
   */
}



// sends out a single clock pulse, independent of SPI transmission
// needed for grayscale LED driver
void SPI_Clock_Strobe(void)
{
   // turn off SPI
   SPICR1 &= ~0x40;
   
   // pulse clock
   PTM |= 0x20;
   Timer_Wait(50);
   PTM &= ~0x20;   
   
   // turn on SPI
   SPICR1 |= 0x40; 
}



// sends 4 bits of 0 out SPI
void SPI_Out_Nibble(void)
{
   // turn off SPI
   SPICR1 &= ~0x40; 
  
   // output all 0
   PTM &= ~MASK_SPI_DATA;
   
   // 4 clock cycles
   PTM |= MASK_SPI_CLOCK;
   PTM &= ~MASK_SPI_CLOCK;
   PTM |= MASK_SPI_CLOCK;
   PTM &= ~MASK_SPI_CLOCK;
   PTM |= MASK_SPI_CLOCK;
   PTM &= ~MASK_SPI_CLOCK;
   PTM |= MASK_SPI_CLOCK;
   PTM &= ~MASK_SPI_CLOCK;     
   
       
   // turn on SPI
   SPICR1 |= 0x40;   
}
   


// output bits for a grayscale LED driver
// outputs in reverse order so that input[0] cooresponds to Ch0
void SPI_GS_Out16(unsigned char *input)
{       
   char j;
   
   for (j=15; j>=0; j--) 
   {	     
      SPI_Out_Nibble();
      
      SPIDR = input[j];
      SPI_WAIT_FOR_DATA;
      dummy = SPIDR;
      
      /*
      SPI_WAIT_FOR_SPACE;
      SPIDR = input[j];	    
      */
   }	     
}

// latches the grayscale registers
void SPI_GS_Latch(void)
{
   // pulse latch
   PTT |= MASK_GS_LATCH;
   PTT &= ~MASK_GS_LATCH;
}
  

// fills Buffer with values of human input pots and switches
// values are 10bit
void SPI_Retrieve_Human_Input(unsigned short *Buffer, unsigned char NumChan)
{
   unsigned char j;
   
   for (j=0; j<NumChan; j++)
   {
      // send channel address to 595
      SPIDR = j;
      SPI_WAIT_FOR_DATA;
      dummy = SPIDR;   
      
      // latch the 595
      PTT |= MASK_SHIFT_LATCH;
      PTT &= ~MASK_SHIFT_LATCH; 

      // wait 1 microsecond, to allow the analog multiplexer to update
      // == 24 clock cycles
      Timer_Wait(70);

      // get audio reading from ATD
      Buffer[j] = ADC_In(HUMAN_INPUT_PIN);
   }
}


// fills Audio with values of input bandpass channels 0 - (NumChan-1)
// sends a value through SPI to the 595 connected to the analog
// multiplexer, then reads the value on the proper port AD pin,
// specified by: AUDIO_INPUT_PIN
void SPI_Retrieve_Audio(unsigned short *Audio, unsigned char NumChan)
{
   unsigned char j;
   
   for (j=0; j<NumChan; j++)
   {
      // send channel address to 595
      SPIDR = j;
      SPI_WAIT_FOR_DATA;
      dummy = SPIDR;   
      
      // latch the 595
      PTT |= MASK_SHIFT_LATCH;
      PTT &= ~MASK_SHIFT_LATCH; 

      // wait 1 microsecond, to allow the analog multiplexer to update
      // == 24 clock cycles
      Timer_Wait(70);  // this is overkill

      // get audio reading from ATD
      Audio[j] = ADC_In(AUDIO_INPUT_PIN);
   }
}
